----------------------------------------------------------------
DC():PrintDebug( "\r\n------ LOADING GAME SERVER V.0.51 ------" )
----------------------------------------------------------------


-- Listeners

dcpp:setListener( "pm", "ChillGameServer",
	function( hub, user, data )
		local nick = user:getNick()
		GameServer:OnPM( hub, nick, data )
	end
)

dcpp:setListener( "timer", "ChillGameServer",
	function()
		GameServer:OnTimer()
	end
)

dcpp:setListener( "userConnected", "ChillGameServer",
	function( hub, user )
		local nick = user:getNick()
		GameServer:OnUserConnected( hub, nick )
	end
)

dcpp:setListener( "connected", "ChillGameServer",
	function( hub )
		GameServer:OnHubConnected( hub )
	end
)

dcpp:setListener( "disconnected", "ChillGameServer",
	function( hub )
		GameServer:OnHubDisconnected( hub )
	end
)





--|||||||||||||||||||||||||--
-----------------------------
-- GAME SERVER ENVIRONMENT --
-----------------------------
--|||||||||||||||||||||||||--


-------------------------------
-- Globals Game Server Table --
-------------------------------

GameServer = {}

-- Data Table
GameServer._Data = {}

-- Settings Table
GameServer._Sets = {}

-- Folder
GameServer._Folder = "scripts/ChillGameServer/"

---------------------
-- Event Functions --
---------------------

-- On Privat Message
function GameServer:OnPM( hub, nick, data )

	-- Parse Data
	GameServer:ParseData( hub, nick, data )

end

-- On Timer ( each second )
function GameServer:OnTimer()

	-- Execute Time Functions
	GameServer:doTFuncs()

end

-- On User Connect
function GameServer:OnUserConnected( hub, nick )

	-- Send On Connect Info
	GameServer:SendConnectInfo( hub, nick )
end

-- On Hub Connect
function GameServer:OnHubConnected( hub )

	-- Assume 5 secs delay is enough to get his own Nick
	self:RegTFunc( os.clock(), 5, { self }, self.CheckMatch )
end

-- On Hub Disconnect
function GameServer:OnHubDisconnected( hub )
end



-- On Main()
function GameServer:Main()

	-- Add GameServer
	GameServer:AddServer( "GS" )

	-- Register Static Clean Chat every 60 seconds
	GameServer:RegStaticTFunc( "CleanChat", 60, { GameServer }, GameServer.CleanChat )

	-- Parse INI file
	GameServer:ParseINI()

end





--/////////////////--
-- GS Game Handler --
--/////////////////--


-- GS Games --
GameServer._Games = {}

-- GS Chats --
GameServer._Chats = {}

-- Chat Timeout in Minutes --
GameServer._ChatTimeout = 10

------------------------
-- Handling Functions --
------------------------

-- SetGame ( GameName, Func to Execute, hub, { players }, invitor )
function GameServer:SetGame( gamename, gamefunc, hub, tplayers, invitor)

	local status = "SETGAME"

	-- Check if anybody is a player
	table.foreach( tplayers, function( _, nick )
		local gname,_ = self:GetPlayer( hub, nick )
		if gname then
			self:SendToNick( hub, invitor, "Failed: "..nick.." is already a player of: "..gname )
			status = "FAILED"
			return 1
		end
	end)

	if (status == "SETGAME") then

		-- Delete user chats & Setup Game References

		local hubid = hub:getId()
		self._Games[hubid] = self._Games[hubid] or {}

		table.foreachi( tplayers, function( _,nick )
			self:DelChat( hub, nick)
			self._Games[hubid][nick] = { gamename, gamefunc }
		end)

		return 1
	end
end

-- GetPlayer ( hub, nick ) returns gamename, gamefunction on success
function GameServer:GetPlayer( hub, nick )
	local hubid = hub:getId()
	if self._Games[hubid] and self._Games[hubid][nick] then
		return self._Games[hubid][nick][1], self._Games[hubid][nick][2]
	end
end

-- DelGame ( hub, { players } )
function GameServer:DelGame( hub, tplayers )

	self:SetChat( hub, tplayers )

	local hubid = hub:getId()

	if self._Games[hubid] then	
		table.foreachi( tplayers, function( _, nick )
			self._Games[hubid][nick] = nil
		end)
	end
end

-- SetChat ( hub, { chatters } )
function GameServer:SetChat( hub, tchatters )

	if (table.getn(tchatters) >= 2) then

		local hubid = hub:getId()

		self._Chats[hubid] = self._Chats[hubid] or {}

		local hubchat = self._Chats[hubid]

		table.foreachi(tchatters, function( _, nick )
			local sendtonicks = self:GetSendTos( nick, tchatters )
			hubchat[nick] = {}
			hubchat[nick]._SendTos = sendtonicks
			hubchat[nick]._Timeout = self._ChatTimeout
		end)
		self:SendToNicks( hub, tchatters, "*** Chat opened ***")
	end
end

-- GetSendTos ( nick, tnicks )
function GameServer:GetSendTos( nick, tnicks )
	local sendtos = {}
	table.foreachi( tnicks, function( _, snick)
		if (snick ~= nick) then
			table.insert( sendtos, snick )
		end
	end)
	return sendtos
end

-- DelChat ( hub , nick )
function GameServer:DelChat( hub, nick )
	local hubid = hub:getId()
	if self._Chats[hubid] and  self._Chats[hubid][nick] then
		
		local sendtos = self._Chats[hubid][nick]._SendTos

		table.foreachi(sendtos, function( _, snick )
			self._Chats[hubid][snick] = nil
		end)

		self._Chats[hubid][nick] = nil

		table.insert( sendtos, nick )
		self:SendToNicks( hub, sendtos, "*** Chat closed ***" )
	end
end

-- CleanChat ( )
function GameServer:CleanChat()
	table.foreach(self._Chats, function( hubid, tnicks )
		table.foreach(tnicks, function( nick, tnick )
			tnick._Timeout = tnick._Timeout - 1
			if (tnick._Timeout <= 0) then
				local hub = dcpp:getHub( hubid )
				self:DelChat( hub, nick )
			end
		end)
	end)
end


--////////////////--
-- GS Environment --
--////////////////--


---------------------
-- Timer Functions --
---------------------

GameServer._TFuncs = {}
GameServer._StaticTFuncs = {}

-- RegTFunc( funcname, execution delay, { arguments }, function )
function GameServer:RegTFunc( fname, delay, args, func )
	if fname
	 and type(delay) == "number"
	 and type(args) == "table"
	 and type(func) == "function" then
		fname = tostring(fname)
		self._TFuncs[fname] = { delay, args, func }
	else
		DC():PrintDebug("Error: RegTFunc: "..fname)
		
	end
end


-- RegStaticTFunc ( funcname, interval, { arguments }, function )
function GameServer:RegStaticTFunc( fname, interv, args, func )
	if fname
	 and type(interv) == "number"
	 and type(args) == "table"
	 and type(func) == "function" then
		fname = tostring(fname)
		self._StaticTFuncs[fname] = { { 0,interv }, args, func }
	else
		DC():PrintDebug("Error: RegTFunc: "..fname)
		
	end
end

-- doTFuncs ( ) execute functions
function GameServer:doTFuncs()

	-- No Static
	table.foreach(self._TFuncs, function( fname, tfunc )
		tfunc[1] = tfunc[1] - 1
		if (tfunc[1] <= 0) then
			-- execute
			tfunc[3](unpack(tfunc[2]))
			self._TFuncs[fname] = nil
		end
	end)

	-- Static
	table.foreach(self._StaticTFuncs, function( fname, tfunc )
		tfunc[1][1] = tfunc[1][1] + 1
		if (tfunc[1][1] >= tfunc[1][2]) then
			-- reset
			tfunc[1][1] = 0
			-- execute
			tfunc[3](unpack(tfunc[2]))
		end
	end)
end

-----------
-- INPUT --
-----------

-- ParseData ( hub, nick, data )
function GameServer:ParseData( hub, nick, data)

	local targs = Targs:Create( data )

	-- check if arguments exists
	if targs:get() then

		-- Check if Trigger command
		if self:OnTrigs( hub, nick, targs ) then
			return 1
		end


		-- Only Funcs for GameServer
		if self:IsHub( hub, "GS" ) then

			local arg1 = targs:getArg( _, 1 )

			if self._GSTrigs[arg1] then
				targs:removeArg()
				self._GSTrigs[arg1]( hub, nick, targs:getArgs() )
				return 1
			end
		
			local _,sfunc = self:GetPlayer( hub, nick )

			-- Is Players
			if sfunc then
				sfunc( hub, nick, targs:getArgs(), data )
				return 1
			end

			-- Check if chatter
			local hubid = hub:getId()
			if self._Chats[hubid] and  self._Chats[hubid][nick] then
				self:SendToNicks( hub, self._Chats[hubid][nick]._SendTos, data, nick)
				return 1
			end
		end
	end
end


-----------------
-- Targs Class --
-----------------
Targs = {}

function Targs:Create( data, token )

	token = token or "%s+"

	local targs = {

		_token = token,
		_data = data,
		_args = {},

		getArg = function( targs, argnum, lower )
			argnum = argnum or 1
			local args = {}
			for i = 1,argnum do
				if targs._args[1] then
					if lower then
						table.insert( args, string.lower( targs._args[1] ) )
					else
						table.insert( args, targs._args[1] )
					end
				else
					break
				end
			end
			return unpack( args )
		end,
		getArgs = function( targs )
			return targs._args
		end,
		getString = function( targs )
			return targs._data
		end,
		get = function( targs, num )
			local num = num or 1
			if (table.getn( targs._args ) >= num) then
				return 1
			end
		end,
		removeArg = function( targs, num )
			local num = num or 1
			for i = 1, num do
				table.remove( targs._args, 1 )
				if targs._data then
					_,_,targs._data = string.find( targs._data, "^[^"..targs._token.."]+"..targs._token.."(.*)" )
				end
			end
		end,

	}

	for arg in string.gfind( data, "([^"..token.."]+)" ) do
		table.insert( targs._args, arg )
	end
	return targs
end


------------
-- Output --
------------

-- SendToNick ( hub, nick, data, fromnick ) Send data to a certain nick
function GameServer:SendToNick( hub, nick, data, fromnick )
	local ownNick = hub:getOwnNick()
	if ownNick then
		if fromnick then
			DC():SendHubMessage( hub:getId(), "$To: "..nick.." From: "..ownNick.." $<"..ownNick.."> "..self._ChatChar.." <"..fromnick.."> "..data.."|" )
		else
			DC():SendHubMessage( hub:getId(), "$To: "..nick.." From: "..ownNick.." $<"..ownNick.."> "..data.."|" )
		end
	end
end

-- SendToNicks ( hub, { nicks }, data, fromnick ) Send data to various nicks
function GameServer:SendToNicks( hub, tnicks, data, fromnick )
	table.foreachi( tnicks, function( _,nick )
		if (nick ~= fromnick) then
			self:SendToNick( hub, nick, data, fromnick )
		end
	end)
end

-- SendConnectInfo
function GameServer:SendConnectInfo( hub, nick )
	if self._Sets.SendInfo and (self._Sets.SendInfo == 1) then
		if self:IsHub( hub ) then
			self:SendToNick( hub, nick, self._Data._Help )
		end
	end
end

------------
-- Others --
------------

-- Token
function GameServer:Tokenize( token, data )
	local targs = {}
	for arg in string.gfind(data,"([^"..token.."]+)") do
		table.insert( targs, arg )
	end
	return targs
end

-- Is Hub
function GameServer:IsHub( hub, sserver )

	local hubid = hub._id

	if self._HubID[hubid] then
		if sserver then
			if self._HubID[hubid]._sets[sserver] then
				return 1
			end
		else
			return 1
		end
	end
end


-- Is Allowed
function GameServer:IsAllowed( hub, nick, prof )

	local tprof = {}

	if ( type(prof) ~= "table" ) then

		prof = tostring(prof)
		tprof[prof] = 1
	else
		tprof = prof
	end

	-- check for Owner
	if tprof["$Owner"] then
		local ownNick = hub:getOwnNick()
		if ownNick and ( ownNick == nick ) then
			return 1
		end
	end

	-- check for OP
	if tprof["$OP"] then
		if shub:isOp(nick) then
			return 1
		end
	end

	-- Check for All
	if tprof["$ALL"] then
		return 1
	end

	-- Check for nick
	if tprof[nick] then	
		return 1
	end

	-- Check for hub
	if tprof["$HUB"] then
		return self:IsHub( hub )
	end

end

-- on Triggers
function GameServer:OnTrigs( hub, nick, targs )

	local arg1 = targs:getArg( _, 1 )
	local ttrig = self._Trigs[arg1]
	if ttrig then
		if self:IsAllowed( hub, nick, ttrig[2] ) then
			targs:removeArg()
			ttrig[1]( self, hub, nick, targs )
			return 1
		end
	end
end

	

--/////////////////////////--
-- GameServer HUBS/Servers --
--/////////////////////////--

-- GS Servers
GameServer._Servers = {}

-- Hubaddy Settings Table
GameServer._HubNicks = {}

-- Hub ID Nicks
GameServer._HubID = {}

--------------
-- Handlers --
--------------

-- Add Server
function GameServer:AddServer( server )
	self._Servers[server] = 1
end

-- ParseINI
function GameServer:ParseINI()

	DC():PrintDebug("\r\n\tParsing INI:\r\n")

	-- Reset Tables
	self._HubNicks = {}
	self._HubID = {}

	local handle = io.open(self._Folder.."GameServer.ini","r")
	if handle then
		for line in handle:lines() do
			local _,_,hubaddy,snick,servers,multiid = string.find(line,"^([^|]+)|([^|]+)|([^|]+)$")
			if snick then
				self:InsertHub( hubaddy, snick, servers, multi )
			end
		end
	end

	DC():PrintDebug("\tFinished Parsing\r\n")

end

-- ServerstoTable
function GameServer:ServerstoTable( servers )
	local tstring = {}
	local sret = nil
	string.gsub( servers, "([^,]+)", function ( server )
		if self._Servers[server] then
			tstring[server] = 1
		else
			sret = server
			return
		end
	end)
	return tstring, sret
end

-- ServerstoString
function GameServer:ServerstoString( tservers )
	local sstring = ""
	table.foreach( tservers, function( serv,_ )
		sstring = sstring..serv..","
	end)
	return sstring
end

-- InsertHub
function GameServer:InsertHub( shubaddy, snick, sservers )
	
	local msg = "\r\n"

	local tservers, serr = self:ServerstoTable( sservers )

	if serr then
		msg = "\t*No Valid Server: "..serr..", in address: "..shubaddy..", servers: "..sservers..", NOT ADDED\r\n"
		DC():PrintDebug(msg)
		return (msg)
	end

	self._HubNicks[shubaddy] = self._HubNicks[shubaddy] or {}

	local dnick,sserv = 0,""

	table.foreach(self._HubNicks[shubaddy], function( cnick, v )
		local tset = v._sets	
		return table.foreachi(tsettings, function( serv, _ )
			if tset[serv] then
				dnick, sserv = cnick, serv
				return 1
			end
		end)
	end)
	
	-- check for double nick			
	if (dnick == 0) then
		msg = "\tAdded: Hub: "..shubaddy..", Nick: "..snick..", Setting: "..sservers.."\r\n"
		self._HubNicks[shubaddy][snick] = { _sets = tservers, _matched = 0 }

		local matched = self:MatchHub( shubaddy, snick, tservers )
		if matched then
			self._HubNicks[shubaddy][snick]._matched = 1
			msg = msg.."\tMatched Hub with "..matched.."\r\n"
		end
	else
		msg = msg.."\t*Double servers, Hub: "..shubaddy..", Nick1: "..dnick..", Nick2: "..snick..", server: "..sserv.."\r\n"
		msg = msg.."\t*Not Added\r\n"
	end
	DC():PrintDebug(msg)
	return (msg)
end

-- MatchHub
function GameServer:MatchHub( shubaddy, snick, tservers )

	local address = self:GetIP(shubaddy)

	if not address then
		DC():PrintDebug("\tError: Couldn't, get IP of "..shubaddy)
		DC():PrintDebug("\tTrying again in 1 min")
		self:RegTFunc(os.clock(), 60, { self, shubaddy, snick, tservers }, self.MatchHub )
		return
	else
		DC():PrintDebug("\tResolved IP of "..shubaddy.." - "..address)
	end

	for _,shub in dcpp:getHubs() do
		local hubid = shub:getId()
		if (shub:getAddress() == address) then
			local ownNick = shub:getOwnNick()
			if ownNick then
				if (ownNick == snick) then
					DC():PrintDebug("\tMatched Hub: "..shub:getHubName().." to "..shubaddy..", Nick: "..snick)
					self._HubID[hubid] = { _addy = shubaddy, _nick = snick, _sets = tservers, _help = self:GetFullHelp( tservers, shubaddy ) }
					return shub:getHubName()
				end
			else
				DC():PrintDebug("\tOwnNick not set: "..shub:getHubName().."  ( "..shubaddy.." )")
				DC():PrintDebug("\tTrying again in 1 min")
				self:RegTFunc(address, 60, { self }, self.CheckMatch )
			end
		end
	end
end


-- GS Get Full Help
function GameServer:GetFullHelp( tset, shubaddy )

	local curhelp = "\r\n\r\n\t-- "..shubaddy.." HELP --\r\n\r\n\tcredits - Shows Credits\r\n\r\n"

	-- Check GS
	if tset["GS"] then
		-- Check Chess
		if RasChessEnv then
			curhelp = curhelp.."\t"..RasChessEnv._Data.help.."\r\n"
		end
		-- Check TTT
		if TTTServ then
			curhelp = curhelp.."\t"..TTTServ.Data.Help.."\r\n"
		end
		-- Check ConnectX
		if ConnectX then
			curhelp = curhelp.."\t"..ConnectX.Data.Help.."\r\n"
		end
		-- Check MADN
		if MADN then
			curhelp = curhelp.."\t"..MADN._Data.Help.."\r\n"
		end
	end
			
	-- Check Triv
	if tset["Trivia"] then
		if TrivEx then
			curhelp = curhelp.."\r\n\t--- Help for Trivia Ex type [+-!]trivhelp ---\r\n"
		end
	end

	-- Check Jumble
	if tset["Jumble"] then
		if JumbleRunFlag then
			curhelp = curhelp.."\r\n\t--- Help for Jumble 2 by RabidWombat ype a.help ---\r\n"
		end
	end

	return curhelp
end

-- GetIP
function GameServer:GetIP(shubaddy)

	-- hubaddy is IP
	if string.find(shubaddy,"^(%d+%.%d+%.%d+%.%d+)") then
		return shubaddy
	end

	local _,_,hostname = string.find(shubaddy,"^([^:]+)")
	local _,_,port = string.find(shubaddy,":(%d+)$")

	os.execute("nslookup "..hostname.." > lookupres.txt")


	local match,ip = 0,nil
	for line in io.lines("lookupres.txt","r") do
		if (match == 1) then
			_,_,ip = string.find(line,"^%S+%s+(%d+%.%d+%.%d+%.%d+)")
			match = 0
		elseif (string.sub(line,1,5) == "Name:") then
			match = 1
		end
	end

	os.remove( "lookupres.txt" )

	if ip then
		local sip = ""
		if port then
			sip = ip..":"..port
		else
			sip = ip
		end
		return sip
	end
end

-- CheckMatch
function GameServer:CheckMatch()
	table.foreach(self._HubNicks, function( shubaddy,v )
		return table.foreach(v, function( snick, v2 )
			local match = v2._matched
			local tservers = v2._sets
			if ( match == 0 ) then
				if self:MatchHub( shubaddy, snick, tservers ) then
					self._HubNicks[shubaddy][snick]._matched = 1
					return 1
				end
			end
		end)
	end)
end

-- RemoveMatch
function GameServer:RemoveMatch( shub )
	local hubid = shub:getId()
	if self._HubID[hubid] then
		local shubaddy = self._HubID[hubid]._addy
		local snick = self._HubID[hubid]._nick
		self._HubID[hubid] = nil

		if self._HubNicks[shubaddy][snick] then
			DC():PrintDebug("\tUnmatched Hub: "..shub:getHubName().." - "..shubaddy..", Nick: "..snick)
			self._HubNicks[shubaddy][snick]._matched = 0
		end
	end
end




--/////////////////////////////////////////////////////////--
-- GS USER FUNCTIONS ( OnTrig ) ( hub, nick, trags, data ) --
--/////////////////////////////////////////////////////////--


-------------
-- Help
-------------
function GameServer:Help( hub, nick )
	self:SendToNick( hub, nick, self._Data._Help)
end

function GameServer:FullHelp( hub, nick )
	local hubid = hub:getId()
	self:SendToNick( hub, nick , self._HubID[hubid]._help)
end

-------------
-- Credits --
-------------
GameServer._Data._Credits = ""..
		"\t****** CREDITS *******\r\n\r\n"..
		"\tBig credits go out to (people I couldn't do without):\r\n"..
		"\t - \"Queeny\" for good ideas and endless testing\r\n"..
		"\t - \"UwVerlosser\" for drawing all the boards in MicroSoftSansSerif 8, Ideas and testing\r\n"

function GameServer:Credits( hub, nick )
	self:SendToNick( hub, nick, self._Data._Credits )
end



---------------------
-- GS CONFIG FUNCS --
---------------------

-------------------
-- Save HubNicks --
-------------------
function GameServer:SaveHubNicks()
	local handle = io.open("scripts/ChillGameServer/GameServer.ini","w")
	table.foreach(self._HubNicks, function( hubaddy,v )
		table.foreach(v, function( snick, v2 )
			local sset = self:ServerstoString(v2._sets)
			handle:write(hubaddy.."|"..snick.."|"..sset.."\n")
		end)
	end)
	handle:close()
end

--------------------
-- GS Config Help --
--------------------
GameServer._Data._ConfigHelp = ""..
	"\t   -- Game Server Config Help --\r\n\r\n"..
	"\tlistservers   -- List available Servers\r\n"..
	"\taddhub <hubaddy> <nick> <servers>   -- Adds entered hub, e.g. 'addhub -GS- chill.ath.cx Server1,Server2...'\r\n"..
	"\tdelhub <hubaddy> <nick> -- Deletes entered hub\r\n"..
	"\tlisthubs   -- Lists all hubs\r\n"..
	"\tconfighelp   -- This help\r\n"

function GameServer:ConfigHelp( hub, nick )
	self:SendToNick( hub, nick, self._Data._ConfigHelp )
end

------------------
-- List Servers --
------------------
function GameServer:ListServers(shub, snick)
	self:SendToNicks(shub,{snick},"Available Servers:")
	local servers = self:ServerstoString(self._Servers)
	self:SendToNicks(shub,{snick},"\t"..servers)
end

-------------
-- Add Hub --
-------------
function GameServer:AddHub( hub, nick, targs )
	if targs:get(3) then
		local hubaddy, inick, servers = targs:getArgs(3)
		local msg = self:InsertHub( hubaddy, inick, servers )
		self:SendToNick( hub, nick, msg )
		self:SaveHubNicks()
	else
		self:ConfigHelp( hub, nick )
	end
end

-------------
-- Del Hub --
-------------
function GameServer:DelHub( hub, nick, targs )
	if targs:get(2) then
		local shubaddy, inick = targs:getArgs(2)
		if self._HubNicks[shubaddy] and self._HubNicks[shubaddy][inick] then
			local match = self._HubNicks[shubaddy][inick]._matched
			if (match == 1) then
				table.foreach(self._HubID, function( id, v )
					local addy = v._addy
					local nick = v._nick
					if (addy == shubaddy) and (nick == inick) then
						self._HubID[id] = nil
						self:SendToNicks(shub,{snick},"Unmatched Hub: \""..addy.."\" Nick: \""..nick.."\"")
					end
				end)
			end
			self._HubNicks[shubaddy][inick] = nil
			self:SaveHubNicks()
			self:SendToNick( hub, nick,"Deleted Hub: \""..shubaddy.."\" Nick: \""..inick.."\"")
		else
			self:SendToNick( hub, nick, "No such Hub: \""..shubaddy.."\" Nick: \""..inick.."\"")
		end
	else
		self:ConfigHelp( hub, nick )
	end
end

---------------
-- List Hubs --
---------------
function GameServer:ListHubs( hub, nick )
	self:SendToNick( hub, nick, "Game Server Hubs:" )
	table.foreach(self._HubNicks, function( hubaddy, v )
		table.foreach( v, function( curnick, v2 )
			local matched = v2._matched
			local sset = self:ServerstoString( v2._sets )
			self:SendToNick ( hub, nick, "Hub: \""..hubaddy.."\", Nick: \""..curnick.."\", Sets: \""..sset.."\", Matched: "..matched )
		end)
	end)
end




--//////////--
-- TRIGGERS --
--//////////--

-- { function, profile }
-- prfile = proftable or "$Owner", "$OP", "$ALL", "$HUB"
GameServer._Trigs = {

	help = { GameServer.Help, "$HUB" },
	fullhelp = { GameServer.FullHelp, "$HUB" },
	credits = { GameServer.Credits, "$HUB" },

	confighelp = { GameServer.ConfigHelp, "$Owner" },
	listservers = { GameServer.ListServers, "$Owner" },
	addhub =  { GameServer.AddHub, "$Owner" },
	delhub =  { GameServer.DelHub, "$Owner" },
	listhubs=  { GameServer.ListHubs, "$Owner" },

}

function GameServer:AddTrig( trig, func, prof )
	if trig
	and (type(func) == "function")
	and prof then
		self._Trigs[tostring(trig)] = { func, prof }
		return 1
	end
end
	

--///////////////////////////////--
-- LOAD OTHER GAMES AND SETTINGS --
--///////////////////////////////--

-- Table with GSTrigs
GameServer._GSTrigs = {}

function GameServer:AddGSTrig( trig, func )
	if trig
	and (type(func) == "function") then
		self._GSTrigs[tostring(trig)] = func
		return 1
	end
end


-- Load Settings and Games
dofile("scripts/ChillGameServer/GameServer.Settings.cfg")



--///////////--
-- CALL MAIN --
--///////////--

GameServer:Main()



---------------------------------------------------------------
DC():PrintDebug( "------ LOADED GAME SERVER V.051 ------\r\n" )
---------------------------------------------------------------