-- MADN by Chill Inc. Server V 002
-- the board
-- 40 fields

-- For the bot to be trgged on Timer
dcpp:setListener( "timer", "MADN",
	function()
		MADN:OnTimer()
	end
)


-- MADN

MADN = {}

-- Timer
MADN._Timer = {}

function MADN:OnTimer()
	table.foreach(self._Timer, function(fname, tfunc)

		tfunc[1] = tfunc[1] - 1
		if (tfunc[1] == 0) then

			local func = tfunc[2]
			local args = tfunc[3]
			func(unpack(args))

		elseif (tfunc[1] < 0) then
			self:DelTimerFunc(fname)
		end
	end)
end

function MADN:AddTimerFunc(when, fname, func, targs)
	if (type(when) == "number")
	 and (type(func) == "function")
	 and (type(targs) == "table") then
		self._Timer[fname] = { when, func, targs }
	end
end

function MADN:DelTimerFunc(fname)
	self._Timer[fname] = nil
end

-- Folder

MADN._Folder = "scripts/ChillGameServer/MADN.Server/"

-- Load Other Files
MADN._DrawBoard = {}

dofile(MADN._Folder.."Fonts/MADN.Text.lua")
dofile(MADN._Folder.."Fonts/MADN.SansSerif.lua")


-- Fonts
MADN._Fonts = {}
MADN._Fonts.Text = 1
MADN._Fonts.SansSerif = 1

MADN._Fonts.default = "SansSerif"


----------------------
-- MADN GAME TABLES --
----------------------
MADN._Games = {}
MADN._Players = {}
-----------------
-- CREATE GAME --
-----------------
function MADN:CreateNewGame(shub, snicks)


	-- Get a New Game Index
	local sindex = ""
	local found = 1
	while found do
		sindex = sindex..string.char(math.random(48,122))
		if not self._Games[sindex] then
			found = nil
		end
	end

	local sgame = {}
	sgame._index = sindex

	-- reference hub
	sgame._hub = shub

	-- The Players
	sgame._Players = {}

	-- The Playnums
	sgame._Playnum = {}

	-- to Play table
	sgame._toPlay = {}

	-- refence the nicks to their player numbers
	table.foreachi(snicks, function(i,v)
		local counter = i - 1
		local snick = v[1]
		local status = v[2]
		sgame._Players[snick] = { _plnum = i }
		table.insert(sgame._Playnum, { _nick = snick, _status = status, _font = self._Fonts.default, _throws = 3, _mode = "rd", _offset = (counter*10) } )
		table.insert(sgame._toPlay,{ _nick = snick, _plnum = i })
		if (status == "Active") then
			self._Players[snick] = sindex
		end
	end)
	
	-- Status Messages
	sgame._statusmsg = ""
	---------------
	-- The Field --
	---------------
	sgame._field = {}

	-- Create Main Field
	for _ = 1,40 do
		-- pos (Possesiong of playernumber, fig (figurenumber of player)
		table.insert(sgame._field,{ _pos = 0 })
	end

	-- table to identify where the fig is by playernumber
	sgame._figs = {}
	for _ = 1,4 do
		table.insert(sgame._figs,{})
	end

	-- table for starts
	sgame._starts = {}
	for i = 1,4 do
		table.insert(sgame._starts,{})
		if snicks[i] then
			for _ = 1,4 do
				-- Insert a fig
				table.insert(sgame._starts[i],1)
			end
		end
	end

	-- table for homes
	sgame._homes = {}
	for _ = 1,4 do
		--leave empty
		table.insert(sgame._homes,{})
	end

	-- the dice
	sgame._dice = 1

	-- Flags setted @ MovFig, Flags: 1-ValidMove, 2-Beat, 3-Home, 4-option on 3 Throws, 5-has 3 throws, 6-Won flag, 7-moved fig, 8-Beaten playersnum
	sgame._flags = { 0, 0, 0, 0, 0, 0, 0, 0 }

	-- Quiet Flag no board/messages are sent
	sgame._quiet = 0

	---------------
	-- Functions --
	---------------
	sgame.SendToPlayers = self._Funcs.SendToPlayers

	sgame.SendToPlayer = self._Funcs.SendToPlayer

	sgame.SendBoard = self._Funcs.SendBoardToPlayers

	sgame.SwitchPlayers = self._Funcs.SwitchPlayers

	sgame.SendStatus = self._Funcs.SendStatus

	sgame.doThrowDice = self._Funcs.doThrowDice
	sgame.ThrowDice = self._Funcs.ThrowDice

	sgame.MoveFig = self._Funcs.MoveFig

	sgame.CheckThrow = self._Funcs.CheckThrow
	sgame.CheckPossibleMoves = self._Funcs.CheckPossibleMoves
	sgame.SetThrowFlags = self._Funcs.SetThrowFlags

	sgame.playAuto = self._Funcs.playAuto
	sgame.checkAutoPlay = self._Funcs.checkAutoPlay

	sgame.ClearField =  self._Funcs.ClearField
	sgame.RemPlayerFig = self._Funcs.RemPlayerFig
	sgame.AddFieldFig = self._Funcs.AddFieldFig
	sgame.MovFieldFig = self._Funcs.MovFieldFig
	sgame.SortFieldFig = self._Funcs.SortFieldFig
	sgame.EnterHome = self._Funcs.EnterHome
	sgame.MoveHome = self._Funcs.MoveHome

	sgame.doMove = self._Funcs.doMove

	sgame.GetFigPos = self._Funcs.GetFigPos

	sgame.GetFieldPos = self._Funcs.GetFieldPos
	sgame.GetIntPos = self._Funcs.GetIntPos

	sgame.AddStatusMsg = self._Funcs.AddStatusMsg

	sgame.DelGame = self._Funcs.DelGame

	-- Set Reference
	self._Games[sindex] = sgame

	return sgame
end

-------------------------
-- Functions for sgame --
-------------------------
MADN._Funcs = {}

function MADN._Funcs.DrawBoardFont(sgame, sfont)
	local board = MADN._DrawBoard[sfont](sgame)
	return board
end

function MADN._Funcs.SendToPlayers(sgame, data, fnick)
	local nicks = {}
	local shub = sgame._hub
	table.foreachi(sgame._Playnum, function(_,v)
		local nick = v._nick
		local status = v._status
		if (status == "Active") then
			if fnick then
				if (nick ~= fnick) then
					table.insert(nicks,nick)
				end
			else
				table.insert(nicks,nick)
			end
		end
	end)
	GameServer:SendToNicks(shub, nicks, data, fnick)
end

function MADN._Funcs.SendToPlayer(sgame, plnum, data)
	if ( sgame._quiet == 0 ) then
		local nick = sgame._Playnum[plnum]._nick
		local shub = sgame._hub
		GameServer:SendToNicks(shub, {nick}, data)
	end
end
	

function MADN._Funcs.SendBoardToPlayers(sgame)
	if ( sgame._quiet == 0 ) then
		local shub = sgame._hub
		table.foreach(sgame._Playnum, function(_,v)
			local nick = v._nick
			local status = v._status
			local sfont = v._font
			if (status == "Active") then
				local board = MADN._Funcs.DrawBoardFont(sgame, sfont)
				GameServer:SendToNicks(shub, {nick}, board)
			end
		end)
	end
end

function MADN._Funcs.SwitchPlayers(sgame)
	local stable = sgame._toPlay[1]
	table.remove(sgame._toPlay, 1)
	table.insert(sgame._toPlay, stable)
end

function MADN._Funcs.SendStatus(sgame)
	if ( sgame._quiet == 0 ) then
		local plnum = sgame._toPlay[1]._plnum
		local player = sgame._toPlay[1]._nick
		local mode = sgame._Playnum[plnum]._mode
		local statusmsg = ""
		if mode == "rd" then
			statusmsg = statusmsg.." Throws: "..sgame._Playnum[plnum]._throws.."\r\n\t* Turn: "..player..", Next Action: Roll the dice ('rd')"
		elseif mode == "mov" then
			statusmsg = statusmsg.." Dice: < "..sgame._dice.." >\r\n\t* Turn: "..player..", Next Action: Move a figure ('mov <num>')"
		end
		local statusmsg = statusmsg..sgame._statusmsg
		sgame:SendToPlayers(" - GAME STATUS -"..statusmsg)
		sgame._statusmsg = ""
	end
	sgame._statusmsg = ""
end

function MADN._Funcs.DelGame(sgame)
	sgame:SendToPlayers("MADN game was deleted")
	local snicks = {}
	table.foreachi(sgame._Playnum, function(_,v)
		local nick = v._nick
		local status = v._status
		if status == "Active" then
			MADN._Players[nick] = nil
			table.insert(snicks,nick)
		end
	end)
	local shub = sgame._hub
	local sindex = sgame._index
	MADN._Games[sindex] = nil
	GameServer:DelGame(shub, snicks)
end

function MADN._Funcs.AddStatusMsg(sgame, msg)
	sgame._statusmsg = sgame._statusmsg.."\r\n\t"..msg
end

--------------
-- The Game --
--------------

--------------------
-- Make One Throw --
--------------------
function MADN._Funcs.doThrowDice(sgame, snick)

	-- Check if its players turn
	if (snick == sgame._toPlay[1]._nick) then

		-- Get Playernumber
		local plnum = sgame._Players[snick]._plnum

		-- Get Playerstable
		local tplnum = sgame._Playnum[plnum]

		local mode = tplnum._mode
		if mode == "rd" then
			sgame:ThrowDice()
			-- decrement Throws
			tplnum._throws = tplnum._throws - 1
			-- Check the Throw
			sgame:CheckThrow(plnum)		
		else
			return "You have to move"
		end
	else
		return "It is not your turn"
	end
end

--------------------
-- Throw the Dice --
--------------------
function MADN._Funcs.ThrowDice(sgame)
	-- Set Random
	sgame._dice = math.random(1,6)
	return sgame._dice
end

-----------------
-- Check Throw --
-----------------
function MADN._Funcs.CheckThrow(sgame, plnum)

	-- Players table
	local tplnum = sgame._Playnum[plnum]

	-- Begin with id dice == 6
	if (sgame._dice == 6) then
		
		tplnum._throws = 1

		-- Check if able to move out of home
		if (table.getn(sgame._starts[plnum]) > 0) then

			-- set quiet
			sgame._quiet = 1

			sgame:MoveFig(plnum, 0, 1)

			-- reset quiet
			sgame._quiet = 0

			-- check moved flag
			if (sgame._flags[1] == 1) then
				sgame:AddStatusMsg("\""..tplnum._nick.."\" rolled a < 6 > and enters the field")
				sgame:SendBoard()
				sgame:SendStatus()
				return 1
			end
		end
	end

	if (sgame:CheckPossibleMoves(plnum)) then
		tplnum._mode = "mov"
		sgame:SendStatus()
	else
		if (tplnum._throws > 0) then
			sgame:AddStatusMsg("\""..tplnum._nick.."\" rolled a ....  < "..sgame._dice.." >")
			sgame:SendStatus()
		else
			sgame:SetThrowFlags(plnum)
			if (sgame._flags[5] == 1) then
				tplnum._throws = 3
			else
				tplnum._throws = 1
			end
			sgame:AddStatusMsg("\""..tplnum._nick.."\" rolled a ....  < "..sgame._dice.." > and cannot move")
			sgame:SwitchPlayers()
			sgame:SendStatus()
			sgame:checkAutoPlay()
		end
	end
end

------------------------------
-- Check for possible moves --
------------------------------
function MADN._Funcs.CheckPossibleMoves(sgame, plnum)

	-- set quiet
	sgame._quiet = 1

	-- Only Checks if there is a possible move

	local cgame = MADN:GetGameCopy(sgame)

	-- Check if can move out of home
	if (cgame._dice == 6) and (table.getn(sgame._starts[plnum]) > 0) then
		cgame:MoveFig(plnum, 0, 1)
		-- check has moved flag
		if (cgame._flags[1] == 1) then
			-- reset quiet
			sgame._quiet = 0
			return 1
		end
	end

	-- Check all other figs in the field
	for i = 1, table.getn(sgame._figs[plnum]) do
		local sfrom = sgame._figs[plnum][i]
		local sto = cgame._dice + sfrom
		cgame:MoveFig(plnum, i, sto)
		if (cgame._flags[1] == 1) then
			-- reset quiet
			sgame._quiet = 0
			return 1
		end
	end

	-- reset quiet
	sgame._quiet = 0

end

---------------------
-- Set Throw Flags --
---------------------
function MADN._Funcs.SetThrowFlags(sgame, plnum)

	-- Check for Possible 3 throws , due to home
	-- Check for 3 throws
	local counter = 0
	for i = 1,3 do
		if sgame._figs[plnum][i] then
			if (sgame._figs[plnum][i] > 40) then
				if  (sgame._figs[plnum][i] == (45-i)) then
					sgame._flags[4] = 1
				else
					sgame._flags[4] = 0
				end
			end
		end
	end
	
	--Check for has 3 throws
	local wcount = 0
	for i = 1,4 do
		if sgame._figs[plnum][i] then
			if  (sgame._figs[plnum][i] == (45-i)) then
				-- set flag has 3 throws
				sgame._flags[5] = 1
				wcount = wcount + 1
			else
				sgame._flags[5] = 0
			end
		end
	end

	-- Set won flag
	if (wcount == 4) then
		sgame._flags[6] = 1
	end

	-- if start is full
	if (table.getn(sgame._starts[plnum]) == 4) then
		sgame._flags[5] = 1
	end
end

-------------
-- do Move --
-------------
function MADN._Funcs.doMove(sgame, snick, targs)

	
	local plnum = sgame._toPlay[1]._plnum

	local sfig = ""

	-- Check if players turn
	if (snick == sgame._toPlay[1]._nick) then
		local mode = sgame._Playnum[plnum]._mode
		if mode == "mov" then
			if not ( targs[1] and tonumber(targs[1]) ) then
				return "No figure given"
			end
		else
			return "You have to roll the dice ('rd')"
		end
	else
		return "It is not your turn"
	end

	-- convert sfig to number
	sfig = tonumber(targs[1])

	local tplnum = sgame._Playnum[plnum]
	
	-- Move Players on start first
	if (table.getn(sgame._starts[plnum]) > 0) then

		local ffrom = sgame:GetFieldPos( 1, plnum )

		-- check if player stand in front of home		
		if (sgame._field[ffrom]._pos == plnum) then

			-- calc new position
			local newpos = 1 + sgame._dice

			local set = 0

			-- Set quiet flag
			if (sgame._quiet == 0) then
				set = 1
				sgame._quiet = 1
			end

			local startfig = table.getn(sgame._figs[plnum])
			sgame:MoveFig( plnum, startfig, newpos )

			-- check has move flag
			if (sgame._flags[1] == 1) then

				-- if fig was not players fig
				if (startfig ~= sfig) then
					sgame:AddStatusMsg("Moved away from start possition first")
				end

				-- reset quiet
				if (set == 1) then
					sgame._quiet = 0
				end

				if (tplnum._throws > 0) then
					tplnum._mode = "rd"
					sgame:SendBoard()
					sgame:SendStatus()
				else
					tplnum._throws = 1
					tplnum._mode = "rd"
					sgame:SwitchPlayers()
					sgame:SendBoard()
					sgame:SendStatus()
					sgame:checkAutoPlay()
				end
				return
			end
			-- reset quiet
			if (set == 1) then
				sgame._quiet = 0
			end
		end
	end

	-- Move players choosen fig
	-- get int players positions

	local plpos = sgame:GetFigPos(plnum, sfig)

	-- Check if valid fignumber
	if plpos then

		-- calc new position
		local newpos = plpos + sgame._dice

		-- try to move the fig
		sgame:MoveFig( plnum, sfig, newpos )

		-- check moved flag
		if (sgame._flags[1] == 1) then

			-- Check won flag
			if (sgame._flags[6] == 1) then
				sgame:SendBoard()
				sgame:SendToPlayers("!!! "..tplnum._nick.." has won this match")
				sgame:DelGame()
			
			else
				
				if (tplnum._throws > 0) then
					tplnum._mode = "rd"
					sgame:SendBoard()
					sgame:SendStatus()
				else
					tplnum._mode = "rd"
					tplnum._throws = 1
					-- check has 3 thorws flag
					if (sgame._flags[5] == 1) then
						tplnum._throws = 3
					end
					sgame:SwitchPlayers()
					sgame:SendBoard()
					sgame:SendStatus()
					sgame:checkAutoPlay()
				end
			end
		else
			sgame:SendToPlayer(plnum, "Couldn't move..., try another fig")
		end
	else
		sgame:SendToPlayer(plnum, "Couldn't fetch figure number "..sfig..", try another fig")
	end
end

function MADN._Funcs.GetFigPos(sgame, plnum, sfig)
	if sgame._figs[plnum][sfig] then
		local plpos = sgame._figs[plnum][sfig]
		return plpos
	end
end

function MADN._Funcs.GetFieldPos(sgame, num, plnum)

	local tplnum = sgame._Playnum[plnum]
	local offset = tplnum._offset

	local fpos = num + offset
	if fpos > 40 then
		fpos = fpos - 40
	end
	return fpos
end

function MADN._Funcs.GetIntPos(sgame, num, plnum)

	local tplnum = sgame._Playnum[plnum]
	local offset = tplnum._offset

	local fpos = num - offset
	if fpos < 1 then
		fpos = 40 - fpos
	end
	return fpos
end

-------------------------------
-- Check if comp should play --
-------------------------------
function MADN._Funcs.checkAutoPlay(sgame)
	local plnum = sgame._toPlay[1]._plnum
	local tplnum = sgame._Playnum[plnum]
	local status = tplnum._status
	if (status == "Passive") then
		MADN:AddTimerFunc(3, sgame._index, sgame.playAuto, {sgame._index, plnum} )
	else
		MADN:DelTimerFunc(sgame._index)
	end
end

function MADN._Funcs.playAuto(gindex, plnum)

	local sgame = MADN._Games[gindex]

	if not sgame then
		return
	end

	local tplnum = sgame._Playnum[plnum]
	local mode = tplnum._mode

	if mode == "rd" then

		sgame:ThrowDice()

		tplnum._throws = tplnum._throws - 1
		
		-- Begin with id dice == 6
		if (sgame._dice == 6) then
		
			tplnum._throws = 1

			-- Check if able to move out of home
			if (table.getn(sgame._starts[plnum]) > 0) then
				sgame:MoveFig(plnum, 0, 1)
				-- check moved flag
				if (sgame._flags[1] == 1) then
					sgame:SendBoard()
					sgame:AddStatusMsg("\""..tplnum._nick.."\" rolled a < 6 > and enters the field")
					sgame:SendStatus()
					sgame:checkAutoPlay()
					return 1
				end
			end
		end

		if (sgame:CheckPossibleMoves(plnum)) then
			tplnum._mode = "mov"
			sgame:SendStatus()
			sgame:checkAutoPlay()
		else
			if (tplnum._throws > 0) then
				sgame:AddStatusMsg("\""..tplnum._nick.."\" rolled a ....  < "..sgame._dice.." >")
				sgame:SendStatus()
				sgame:checkAutoPlay()
			else
				sgame:SetThrowFlags(plnum)
				if (sgame._flags[5] == 1) then
					tplnum._throws = 3
				else
					tplnum._throws = 1
				end
				sgame:AddStatusMsg("\""..tplnum._nick.."\" rolled a ....  < "..sgame._dice.." > and cannot move")
				sgame:SwitchPlayers()
				sgame:SendStatus()
				sgame:checkAutoPlay()
			end
		end

	elseif mode == "mov" then

		-- Move Players on start first
		if (table.getn(sgame._starts[plnum]) > 0) then

			local ffrom = sgame:GetFieldPos( 1, plnum )

			if (sgame._field[ffrom]._pos == plnum) then

				local startfig = table.getn(sgame._figs[plnum])

				local newpos = 1 + sgame._dice
				sgame:MoveFig( plnum, startfig, newpos )

				if (sgame._flags[1] == 1) then
					if (tplnum._throws > 0) then
						tplnum._mode = "rd"
						sgame:SendBoard()
						sgame:SendStatus()
						sgame:checkAutoPlay()
					else
						tplnum._mode = "rd"
						tplnum._throws = 1
						sgame:SwitchPlayers()
						sgame:SendBoard()
						sgame:SendStatus()
						sgame:checkAutoPlay()
					end
					return
				end
			end
		end

		-- Get a GameCopy

		local tposflags = {}

		for i = 1, table.getn(sgame._figs[plnum]) do

			local cgame = MADN:GetGameCopy(sgame)

			sfrom = cgame._figs[plnum][i]
			local sto = cgame._dice + sfrom
			
			cgame:MoveFig(plnum, i, sto)

			if cgame._flags[1] == 1 then
				table.insert(tposflags,{ { i, sto }, cgame._flags })
			end
		end

		-- Decide where to move

		local domove = 0
		local tomove = {}

		-- check for home
		if domove == 0 then
			table.foreachi(tposflags, function(i,v)
				local posfig = v[1]
				local flags = v[2]
				if flags[3] == 1 then
					domove = 1
					table.insert(tomove, posfig)
					return
				end
			end)
		end

		-- check for beat
		if domove == 0 then
			table.foreachi(tposflags, function(i,v)
				local posfig = v[1]
				local flags = v[2]
				if flags[2] == 1 then
					domove = 1
					table.insert(tomove, posfig)
					return
				end
			end)
		end

		-- option on 3 throws
		if domove == 0 then
			table.foreachi(tposflags, function(i,v)
				local posfig = v[1]
				local flags = v[2]
				if flags[4] == 1 then
					domove = 1
					table.insert(tomove, posfig)
					return
				end
			end)
		end

		-- else random
		if domove == 0 then
			local movnum = math.random(1, table.getn(tposflags))
			local posfig = tposflags[movnum][1]
			table.insert(tomove, posfig)
		end

		-- do move
		sgame:MoveFig(plnum, tomove[1][1], tomove[1][2])
		
		if (sgame._flags[6] == 1) then
			sgame:SendBoard()
			sgame:SendToPlayers("!!! "..tplnum._nick.." has won this match")
			sgame:DelGame()
		else
			if (tplnum._throws > 0) then
				tplnum._mode = "rd"
				sgame:SendBoard()
				sgame:SendStatus()
				sgame:checkAutoPlay()
			else
				tplnum._mode = "rd"
				tplnum._throws = 1
				if (sgame._flags[5] == 1) then
					tplnum._throws = 3
				end
				sgame:SwitchPlayers()
				sgame:SendBoard()
				sgame:SendStatus()
				sgame:checkAutoPlay()
			end
		end
	end
end

----------------------------------------------------
-- Move Fig - from, to are internal palyer values --
----------------------------------------------------
function MADN._Funcs.MoveFig(sgame, plnum, fignum, to)

	-----------------
	-- GAME CHECKS --
	-----------------
	local tplnum = sgame._Playnum[plnum]
	if not tplnum then
		return
	end

	local from = 0
	if (fignum ~= 0) then
		if sgame._figs[plnum][fignum] then
			from = sgame._figs[plnum][fignum]
		else
			return
		end
	end

	--------------------
	-- Trying to move --
	--------------------

	--reset flags
	sgame._flags = { 0, 0, 0, 0, 0, 0, 0, 0 }


	-- get field positons
	local fto = sgame:GetFieldPos( to, plnum )
	local ffrom = sgame:GetFieldPos( from, plnum )


	-- check if field is not of playertype self or to = in home
	if (sgame._field[fto]._pos ~= plnum) or (to > 40) then

		-- Leaving Start
		if from == 0 then

			-- Clear first
			sgame:ClearField(plnum, fto)

			-- Add a new figure to field
			sgame:AddFieldFig( plnum, fto )

			-- Set ValidMove Flag
			sgame._flags[1] = 1

		-- Playing in Field
		elseif (from < 40) and (to <= 40) then

			-- Clear first
			sgame:ClearField(plnum, fto)

			-- Move Figure in Field
			sgame:MovFieldFig(plnum, fignum, ffrom, fto, to)

			-- Set ValidMove Flag
			sgame._flags[1] = 1

		-- Leaving Field entering Home
		elseif (from <= 40) and (to > 40) then

			-- get home distance
			local hnum = to - 40

			-- if distcance is equal or lower 4 then correct
			if hnum <= 4 then

				-- make sure to not jump in home
				for i = 1,(hnum-1) do
					-- Cannot Jump in Home
					if sgame._homes[plnum][i] then
						sgame:SendToPlayer(plnum, "You cannot jump in your Home")
						return
					end
				end
				if not sgame._homes[plnum][hnum] then

					-- Entering home :)
					sgame:EnterHome(plnum, fignum, ffrom, to, hnum)

				else
					sgame:SendToPlayer(plnum, "You cannot beat yourself")
					return
				end
			else
				sgame:SendToPlayer(plnum, "You cannot move out of Home")
				return
			end

			-- Set ValidMove Flag
			sgame._flags[1] = 1
			-- Set HomeFlag
			sgame._flags[3] = 1
			sgame:AddStatusMsg("\""..tplnum._nick.."\" made it home")

		-- Moving in Home
		elseif (from > 40) and (to > 40) then

			-- check distance
			local hfrom = from - 40
			local hto = to - 40

			if hto <= 4 then
				for i = (hfrom+1),(hto-1) do
					-- Cannot Jump in Home
					if sgame._homes[plnum][i] then
						sgame:SendToPlayer(plnum, "You cannot jump in your Home")
						return
					end
				end

				if not sgame._homes[plnum][hto] then
					-- Move in Home
					sgame:MoveHome(plnum, fignum, to, hfrom, hto)
				else
					sgame:SendToPlayer(plnum, "You cannot beat yourself")
					return
				end
			else
				sgame:SendToPlayer(plnum, "You cannot move out of your Home")
				return
			end
			-- Set ValidMove Flag
			sgame._flags[1] = 1
		end

		-- check if someone was beaten
		if (sgame._flags[2] == 1) then
			local beatennum = sgame._flags[8]
			local tbeatplnum = sgame._Playnum[beatennum]
			sgame:SetThrowFlags(beatennum)
			-- check if has 3 throws
			if (sgame._flags[5] == 1) then
				tbeatplnum._throws = 3
			end
		end

		-- Set Throw Flags
		sgame:SetThrowFlags(plnum)

	else
		sgame:SendToPlayer(plnum, "You cannot beat yourself")
	end
end

function MADN._Funcs.ClearField(sgame, plnum, fnum)
	if (sgame._field[fnum]._pos ~= 0) and (sgame._field[fnum]._pos ~= plnum) then
		
		-- Set Beaten flag
		sgame._flags[2] = 1

		local beaten = sgame._field[fnum]._pos

		-- remove figure from field
		sgame:RemPlayerFig( beaten, fnum)
		
		-- Set flag beaten plnum
		sgame._flags[8] = beaten

		local nickbeaten = sgame._Playnum[beaten]._nick
		local beater = sgame._Playnum[plnum]._nick
		sgame:AddStatusMsg("\""..beater.."\" kicks one of \""..nickbeaten.."'s\" lads right home")
	end
	sgame._field[fnum] = { _pos = 0 }
end

function MADN._Funcs.RemPlayerFig(sgame, plnum, fnum)

	--DC():PrintDebug("1To Kill: Field   "..fnum)

	local fignum = table.foreachi(sgame._figs[plnum], function( sfig, ipos)
		if (ipos <= 40) then
			local fieldpos = sgame:GetFieldPos( ipos, plnum)
			if (fieldpos == fnum) then
				return sfig
			end
		end
	end)

	if sgame._figs[plnum][fignum] then

		--DC():PrintDebug("2Kill: "..plnum.."   "..fignum)

		table.remove(sgame._figs[plnum],fignum)
		-- insert one fig on beaten start
		table.insert(sgame._starts[plnum],1)
	end
end

function MADN._Funcs.AddFieldFig(sgame, plnum, fnum)
	-- Remove one from start
	table.remove(sgame._starts[plnum])

	-- Add one to players field with internal playnumber
	table.insert(sgame._figs[plnum], 1)

	-- Add a new field figure
	local fignum = table.getn(sgame._figs[plnum])
	sgame._field[fnum] = { _pos = plnum, _fig = fignum }

	-- Setting move fig flag to moved fig
	sgame._flags[7] = fignum
end
		
function MADN._Funcs.MovFieldFig(sgame, plnum, sfig, ffrom, fto, pto)

	-- check if fig actually exists
	if sgame._figs[plnum][sfig] then

		-- set to new players value
		sgame._figs[plnum][sfig] = pto

		-- sort figs
		sgame:SortFieldFig( plnum )

		-- Clean from field
		sgame._field[ffrom] = { _pos = 0 }

		-- Update New field
		sgame._field[fto] = { _pos = plnum }

		-- Setting move fig flag to moved fig
		sgame._flags[7] = sfig
	end
end	

function MADN._Funcs.SortFieldFig(sgame, plnum, val)
	table.sort(sgame._figs[plnum], function(a,b)  return a > b end)
end

function MADN._Funcs.EnterHome(sgame, plnum, sfig, ffrom, pto, phto)

	-- check if fig actually exists
	if sgame._figs[plnum][sfig] then

		-- clear field
		sgame._field[ffrom] = { _pos = 0 }

		-- set to new players value
		sgame._figs[plnum][sfig] = pto

		-- sort
		sgame:SortFieldFig(plnum)

		-- Set home
		sgame._homes[plnum][phto] = 1
	end
end

function MADN._Funcs.MoveHome(sgame, plnum, sfig, pto, pfhome, pthome)

	
	-- check if fig actually exists
	if sgame._figs[plnum][sfig] then

		-- set to new players value
		sgame._figs[plnum][sfig] = pto

		-- sort
		sgame:SortFieldFig(plnum)

		-- clean home
		sgame._homes[plnum][pfhome] = nil

		-- Set home
		sgame._homes[plnum][pthome] = 1
	end
end	
	

----------------
-- Parse Data --
----------------
function MADN.ParseData(shub, snick, targs, data)

	if targs[1] then
		arg1 = string.lower(targs[1])
		if MADN._Trigs[arg1] then
			table.remove( targs, 1 )
			local ret = MADN._Trigs[arg1]( shub, snick, targs )
			if ret then
				GameServer:SendToNick( shub, snick, ret )
			end
			return 1
		end
	end
	local sgame = MADN:GetGame(snick)
	if sgame then
		sgame:SendToPlayers(data, snick)
	end
end

--------------
-- Triggers --
--------------
MADN._Trigs = {}

-- Invite
MADN._Trigs.invite = function(shub, snick, targs)

	if targs[1] then
		local tnicks = MADN:Token(targs[1],",")
		if (table.getn(tnicks) <= 3) then
			local tplayers = {}
			local tchats = {}
			table.insert(tplayers, { snick, "Active" } )
			table.insert(tchats,snick)
			local ret = table.foreachi(tnicks, function(i,nick)
				if nick == snick then
					return "You cannot inivte yourself"
				elseif nick == "comp" then
					table.insert(tplayers, { "bot"..i, "Passive" } )
				elseif not shub._users[nick] then
					return "Nick: "..nick.." is not in hub"
				else
					table.insert(tplayers, { nick, "Active" } )
					table.insert(tchats,nick)
				end
			end)
			if ret then
				return ret
			end
			table.insert(tnicks, 1, snick)
			local ret2 = GameServer:SetGame("MADN", MADN.ParseData, shub, tchats, snick)
			if ret2 then
				local sgame = MADN:CreateNewGame(shub, tplayers)
				sgame:SendBoard()
				sgame:SendStatus()
			end
		else
			return "Cannot invite more than 3 players"
		end
	end
end

-- Trow the dice
MADN._Trigs.rd = function( shub, snick )
	local sgame = MADN:GetGame(snick)
	if sgame then
		local ret = sgame:doThrowDice(snick)
		if ret then
			return ret
		end
	end
end 

-- Trow the dice
MADN._Trigs.mov = function( shub, snick, targs )
	local sgame = MADN:GetGame(snick)
	if sgame then
		local ret = sgame:doMove(snick, targs)
		if ret then
			return ret
		end
	end
end

-- change the font
MADN._Trigs.font = function(shub, snick, data)
	local sgame = MADN:GetGame(snick)
	if sgame then
		local plnum = sgame._Players[snick]._plnum
		local _,_,font = string.find(data,"^%S+%s+(%S+)")
		if font and MADN._Fonts[font] then
			sgame._Playnum[plnum]._font = font
			local board = MADN._Funcs.DrawBoardFont(sgame, font)
			sgame:SendToPlayer(plnum, board)
			sgame:SendToPlayer(plnum, "Change Font to: "..font)
		else
			sgame:SendToPlayer(plnum, "No font or incorrect font")
		end
	end
end	

-- quit
MADN._Trigs.quit = function(shub, snick)
	local sgame = MADN:GetGame(snick)
	if sgame then
		sgame:SendToPlayers(snick.." has quitted the game")
		sgame:DelGame()
	end
end

-- help
MADN._Trigs.help = function(shub, snick)
	GameServer:SendToNicks(shub, { snick }, MADN._Data.Help)
end

----------------
-- MADN Funcs --
----------------
function MADN:Token(data, token)
	local tWords = {}
	local matcher = "([^"..token.."]+)"
	string.gsub(data, matcher, function (w) table.insert(tWords,w) end)
	return tWords
end

function MADN:GetGame(nick)
	if self._Players[nick] then
		local sindex = self._Players[nick]
		local sgame = self._Games[sindex]
		return sgame
	end
end

function MADN:GetGameCopy(t, lookup_table)
	local copy = {}
	for i,v in t do
		-- the mod
		if i == "_hub" then
			copy[i] = v
		elseif type(v) ~= "table" then
			copy[i] = v
		else
			lookup_table = lookup_table or {}
			lookup_table[t] = copy
			if lookup_table[v] then
				copy[i] = lookup_table[v] -- we already copied this table. reuse the copy.
			else
				copy[i] = self:GetGameCopy(v,lookup_table) -- not yet copied. copy it.
			end
		end
	end
	return copy
end

-- Data
MADN._Data = {}
MADN._Data.Help = "--- MADN Help ( by chill ) ---\r\n\r\n"..
	"\t-----------------------------------\r\n"..
	"\tmadn invite <nick1,nick2,...>    - Invite a user/users (if nick = \"comp\", then a bot is added)\r\n"..
	"\tfont <font>    - Change the font ( \"SansSerif\",\"Text\" )\r\n"..
	"\tquit    - Quit the current game\r\n"..
	"\thelp    - Shows you MADN help\r\n"..
	"\t-----------------------------------\r\n"



--------------------------
-- Add Game Server Trig --
--------------------------
GameServer:AddGSTrig( "madn", MADN.ParseData )

-------------------------------------------------------------------------
DC():PrintDebug( "  ** Loaded MADN.V.003.lua **" )
-------------------------------------------------------------------------