-- Chess Environment
-- Load needed Tables and Environment

-- RasChess Table
RasChessEnv = {}

RasChessEnv.Version = "002 Server"
RasChessEnv.Author = "chill"

-- Folder for Main Script files
RasChessEnv.mfolder = "scripts/ChillGameServer/RasChess.Server/"

-- Folder for Fonts
RasChessEnv.ffolder = "scripts/ChillGameServer/RasChess.Server/Fonts/"

-- Table for loading games
--chesstable = {}


-- Load Needed Files
function RasChessEnv:LoadFiles()

	-- Load RasChess Game Table
	dofile(RasChessEnv.mfolder.."RasChess.Game.lua")

	-- Load RasChess Moves
	dofile(RasChessEnv.mfolder.."RasChess.Moves.lua")

	-- Load Fonts
	dofile(RasChessEnv.ffolder.."RasChess.Text.lua")
	dofile(RasChessEnv.ffolder.."RasChess.SansSerif.lua")

	-- Gsub Figures (| to strchar(166))
	self:GsubFigs(RasChess._Text)
	self:GsubFigs(RasChess._SansSerif)
end

-- Function To Gsub the Figures | to strchar(166)
function RasChessEnv:GsubFigs(stable)
	table.foreach(stable.White, function(fig,v)
		table.foreachi(v, function(i,str)
			v[i] = string.gsub(str,"|",string.char(166))
		end)
	end)
	table.foreach(stable.Black, function(fig,v)
		table.foreachi(v, function(i,str)
			v[i] = string.gsub(str,"|",string.char(166))
		end)
	end)
end

-- Creat Triggs to convert chars to numbers
function RasChessEnv:CreateChessTrigs()

	self._GTrigs = {}
	
	self._GTrigs = {}
	self._GTrigs.Lines = {}
	self._GTrigs.Chinks = {}
	for i = 1,8 do
		self._GTrigs.Lines[tostring(i)] = i
	end
	self._GTrigs.Chinks.A = 8
	self._GTrigs.Chinks.B = 7
	self._GTrigs.Chinks.C = 6
	self._GTrigs.Chinks.D = 5
	self._GTrigs.Chinks.E = 4
	self._GTrigs.Chinks.F = 3
	self._GTrigs.Chinks.G = 2
	self._GTrigs.Chinks.H = 1
end

function RasChessEnv:LoadEnv()
	RasChessEnv:LoadFiles()
	RasChessEnv:CreateChessTrigs()
end
RasChessEnv:LoadEnv()

---------------------------------------------------
---------------------------------------------------
--	*** CHESS FUNCTIONS ***
---------------------------------------------------
---------------------------------------------------
RasChessEnv._Data = {}

RasChessEnv._Data.help = "--- CHESS V "..RasChessEnv.Version.." by "..RasChessEnv.Author.." ---\r\n\r\n"..
	"\t-----------------------------------\r\n"..
	"\tchess invite <nick>   - Invite somebody to play chess\r\n"..
	"\tshowboard   - Shows the current session\r\n"..
	"\tback   - Undo your last move\r\n"..
	"\tfont <Font>  - Either \"Text\" or \"SansSerif\"\r\n"..
	"\tquit   - Ends a chess session.\r\n"..
	"\thelp   - Help for Chess\r\n"..
	"\tcredits  - Shows Credits.\r\n\r\n"..
	"\tMove a figure type: \"from to\", e.g. A2 C1\r\n\r\n"..
	"\tFonts:\r\n"..
	"\t - Microsoft Sans Serif\r\n"..
	"\tText Fonts:\r\n"..
	"\t - Courier\r\n"..
	"\t - Courier New\r\n"..
	"\t - Lucida Console\r\n"..
	"\t-----------------------------------\r\n"

RasChessEnv._Data.credits = " -- CHESS CREDITS --\r\n\r\n"..
	"\t\tDeep respect to \"RabidWombat\" for helping me on how to correctly copy a table, I would still be sitting here and wondering,\r\n"..
	"\t\t\tand the Serialization, which I modded ;)\r\n"..
	"\t\tThx to \"UwVerlosser\" for the SansSerif font, looks great :)\r\n"



RasChessEnv._Trigs = {}

RasChessEnv._Trigs.invite = function( hub, nick, targs )
	if not RasChess:GetGame(nick) then
		local invited = targs[1]
		if invited and hub._users[invited] and (invited ~= nick) then
			local ret = GameServer:SetGame("Chess", RasChessEnv.ChessParseData, hub, {nick,invited}, nick)
			if ret then
				local sgame = RasChess:NewGame(hub,invited,nick)
				sgame:SendBoard()
				sgame:SendToPlayers(invited.." beginns. Type \"help\" for help. Type \"quit\" to abort the game.")
			end
		end
	end
end

RasChessEnv._Trigs.quit = function( hub, nick )

	local sgame = RasChess:GetGame(nick)
	if sgame then
		sgame:SendToPlayers(nick.." has quit the game.")
		RasChess:DelGame(nick)
	end
end

RasChessEnv._Trigs.help = function( hub, nick )

	DC():SendHubMessage( hub:getId(), "$To: "..nick.." From: "..hub:getOwnNick().." $<"..hub:getOwnNick().."> "..RasChessEnv._Data.help.."|" )
end

RasChessEnv._Trigs.showboard = function( hub, nick )
	local sgame = RasChess:GetGame(nick)
	if sgame then
		local board = sgame:GetBoard(sgame.Font[nick])
		if (nick == sgame.sides.Top) then
			sgame:SendToPlayerTop(board.."\r\n\tIt's "..sgame.players[1].."'s turn.")
		else
			sgame:SendToPlayerBot(board.."\r\n\tIt's "..sgame.players[1].."'s turn.")
		end
	end
end


RasChessEnv._Trigs.back = function( hub, nick )
	local sgame = RasChess:GetGame(nick)
	if sgame then
		local ret = sgame:UndoLastMove(nick)
		if ret then
			sgame:SendBoard()
			sgame:SendToPlayers("  "..nick.." moved back.")
		else
			if (nick == sgame.sides.Top) then
				sgame:SendToPlayerTop("Last move made by "..sgame.players[2]..",you can only move back your own moves.")
			else
				sgame:SendToPlayerBot("Last move made by "..sgame.players[2]..",you can only move back your own moves.")
			end
		end
	end
end


RasChessEnv._Trigs.font = function( hub, nick, trags )
	local sgame = RasChess:GetGame(nick)
	if sgame then

		local sfont = targs[1]
		if sfont then
			local ret = sgame:SetFont(nick,sfont)
			if ret then
				if (nick == sgame.sides.Top) then
					sgame:SendToPlayerTop("Setted font to: "..sfont)
				else
					sgame:SendToPlayerBot("Setted font to: "..sfont)
				end
			else
				GameServer:SendToNicks(hub,{nick},RasChess.botname.." "..sfont.." is not available.")
			end
		end
	end
end

RasChessEnv._Trigs.credits = function(msg,hub,nick)
	DC():SendHubMessage( hub:getId(), "$To: "..nick.." From: "..hub:getOwnNick().." $<"..hub:getOwnNick().."> "..RasChessEnv._Data.credits.."|" )
end


function RasChessEnv.ChessParseData( hub, nick, targs, msg )

	if targs[1] then
		arg1 = string.lower(targs[1])
		if RasChessEnv._Trigs[arg1] then
			table.remove( targs, 1 )
			RasChessEnv._Trigs[arg1]( hub, nick, targs )
			return 1
		else
			local ret = RasChessEnv:ParseData( nick, targs )
			if (not ret) then
				local sgame = RasChess:GetGame(nick)
				if sgame then
					if (nick == sgame.sides.Top) then
						sgame:SendToPlayerBot(msg,1)
					else
						sgame:SendToPlayerTop(msg,1)
					end
					return 1
				end
			else
				return 1
			end
		end
	end
end

-----------------------------------------------
--	PARSE DATA
-----------------------------------------------
function RasChessEnv:ParseData( nick, targs )

	local from2,from1,to2,to1 = self:GetCoordinates( targs )
	if from1 and from2 and to1 and to2 then


		local sgame = RasChess:GetGame(nick)

		if sgame then

			local tofig = targs[3]
			if tofig then
				tofig = string.sub(tofig,1,1)..string.lower(string.sub(tofig,2,string.len(tofig)))
			end

			-- create move message
			local moveMSG = RasChessEnv:MsgOnMove(sgame,from1,from2,to1,to2,nick)

			-- Check if Move is allowed
			local curgame,var,errl,errc = sgame:Move(from1,from2,to1,to2,nick,1,tofig)

			sgame = curgame

			-- CHECK RESULTS
			if var == "MOVED" then

				sgame:SendBoard()

				sgame:SendToPlayers(moveMSG)

			elseif var == "CHECK" then

				moveMSG = moveMSG.."\t"..nick.." sets "..RasChess._Players[nick].playswith.." to CHECK.\r\n"

				sgame:SendBoard()
				
				sgame:SendToPlayers(moveMSG)

			elseif var == "CHECKMATE" then

				moveMSG = moveMSG.."\t"..nick.." sets "..RasChess._Players[nick].playswith.." to CHECKMATE.\r\n"

				sgame:SendBoard()
				
				sgame:SendToPlayers(moveMSG)

				sgame:DelGame(nick)

			elseif errl then

				sgame:SendToPlayers(nick..", Failed: "..self._MMChinkChar[errc]..errl)
				
			end
		end

		return "Parsed"

	end
end

function RasChessEnv:GetCoordinates( targs )

	if targs[2] then

		from = string.upper(targs[1])
		to = string.upper(targs[2])

		local from2, from1 = self._GTrigs.Chinks[string.sub(from,1,1)],self._GTrigs.Lines[string.sub(from,2,2)]
		local to2, to1 = self._GTrigs.Chinks[string.sub(to,1,1)],self._GTrigs.Lines[string.sub(to,2,2)]

		return from2, from1, to2, to1
	end
end
	
	

function RasChessEnv:MsgOnMove(sgame,from1,from2,to1,to2,snick)
	local msg = "\r\n\r\n\t"..snick.." moves "..sgame[from1..from2].ftype[1].." from "..self._MMChinkChar[tonumber(from2)]..from1.." to "..self._MMChinkChar[tonumber(to2)]..to1..".\r\n"
	if (sgame[to1..to2].posession == sgame.players[2]) then
		msg = msg.."\t"..sgame[from1..from2].ftype[1].." beats "..sgame[to1..to2].ftype[1].."\r\n"
	end
	return(msg)
end

-- Conversion Table NumChink to Char
RasChessEnv._MMChinkChar = {
	[1] = "H",
	[2] = "G",
	[3] = "F",
	[4] = "E",
	[5] = "D",
	[6] = "C",
	[7] = "B",
	[8] = "A",
}

---------------------------------------------------------------------------------------

--	Write Tables

---------------------------------------------------------------------------------------
function OUTWriteTable(table,tablename,file)
	local handle = io.open(file,"w")
	Serialize(table,tablename,handle)
  	handle:close()
end
--------------------------------------------
function OUTSerialize(tTable,sTableName,hFile,sTab)
	sTab = sTab or "";
	hFile:write(sTab..sTableName.." = {\n");
	for key,value in tTable do
		if (type(value) ~= "function") and (key ~= "hub") then
			local sKey = (type(key) == "string") and string.format("[%q]",key) or string.format("[%d]",key);
			if(type(value) == "table") then
				Serialize(value,sKey,hFile,sTab.."\t");
			else
				local sValue = (type(value) == "string") and string.format("%q",value) or tostring(value);
				hFile:write(sTab.."\t"..sKey.." = "..sValue);
			end
			hFile:write(",\n");
		end
	end
	hFile:write(sTab.."}");
end


--------------------------
-- Add Game Server Trig --
--------------------------
GameServer:AddGSTrig( "chess", RasChessEnv.ChessParseData )

DC():PrintDebug( "  ** Loaded Chess.Server.lua **" )